// EYETRACKER - John and Michael ECE 4760

#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <util/delay.h>

#include "trtSettings.h"
#include "trtkernel_1284.c"
#include <stdio.h>
#include <avr/sleep.h>
#include "i2cmaster.h"
#include "twimaster.c"

// serial communication library
// Don't mess with the semaphores
#define SEM_RX_ISR_SIGNAL 1
#define SEM_STRING_DONE 2 // user hit <enter>

#include "trtUart.h"
#include "trtUart.c"
// UART file descriptor
// putchar and getchar are in uart.c
FILE uart_str = FDEV_SETUP_STREAM(uart_putchar, uart_getchar, _FDEV_SETUP_RW);

// semaphore to protect shared variable
#define SEM_SHARED 3
int args[3] ;

#include "txrx_usart1_base.c"
//-----------
// Gryo Defs
//-----------
#define ITG3200W 0xD2
#define ITG3200R 0xD3

#define WHO_AM_I_REG    0x00
#define SMPLRT_DIV_REG  0x15
#define DLPF_FS_REG     0x16
#define INT_CFG_REG     0x17
#define INT_STATUS      0x1A
#define TEMP_OUT_H_REG  0x1B
#define TEMP_OUT_L_REG  0x1C
#define GYRO_XOUT_H_REG 0x1D
#define GYRO_XOUT_L_REG 0x1E
#define GYRO_YOUT_H_REG 0x1F
#define GYRO_YOUT_L_REG 0x20
#define GYRO_ZOUT_H_REG 0x21
#define GYRO_ZOUT_L_REG 0x22
#define PWR_MGM_REG     0x3E

// ADC Variables and Defs
#define CHANNEL0 0
#define CHANNEL1 1
#define CHANNEL2 2
#define CHANNEL3 3
#define CHANNEL4 4
#define CHANNEL5 5
#define CHANNEL6 6
#define CHANNEL7 7
#define NUM_SAMPLES 50

#define data_length 8 // 0: x, 2: y, 4: z, 6: avg0, 7: avg1
#define tx_id 3
char data[data_length];

volatile int sensor_out;
volatile int ADC_out;
volatile int ADC_channel;

// Getting ADC Reference
volatile uint8_t ADC_result0_avg[NUM_SAMPLES];
volatile uint8_t ADC_result1_avg[NUM_SAMPLES];
volatile uint8_t ref_samples;
uint8_t gyro_updated;


// Function Prototypes
void initialize(void);
unsigned char gyro_get(unsigned char);
void gyro(void);
void i2cSetBitrate(unsigned short bitrateKHz);
void init_ADC(void);
void read_ADC(void);
void sensor_task(void);
void get_ADC_reference(void);
void transmit(void);
char avg_array(uint8_t a[], uint8_t num_elements);

ISR (ADC_vect) {
	ADMUX &= 0b11110000;	// Reset MUX
	if (ref_samples < NUM_SAMPLES)
	{
	    if (ADC_channel ==0)
	    {
	   	    ADC_result0_avg[ref_samples] = ADCH;
	   	    ADMUX |= (1<<MUX0);
	   	    ADC_channel = 1;
	    }
	    else if (ADC_channel==1)
	    {
	    	ADC_result1_avg[ref_samples++] = ADCH;
	 	    ADMUX &= ~(1<<MUX0);
		    ADC_channel = 0;
	    }
		ADCSRA |= (1 << ADSC); // start another conversion
	}
    //ADMUX = (15 << 4) & ADC_channel;
}

void initialize(void) {
	//1 = output, 0 = input
	DDRC = 0b00000011; //Port C1, C0 Output
	PORTC = 0b00000000;

	DDRD = 0x10000010; //Port D1 Output (TX), D7 Output (LED)
	PORTD = 0x10000010;
	txrx_init(1,0,249,1);//TX only - 4000 baud - led on

	i2cSetBitrate(40);
	TWCR |= (1<<TWEN);
	gyro_updated = 0;
	init_ADC();
}

void init_ADC(void) {
  // ADC
  // Vref = 5V
  // ADC clock = 16 MHz / 128 = 125 kHz

  // set Vref to AVCC, left-adjust result into ADCH
  ADMUX = ( 1 << ADLAR ) | ( 1 << REFS0 );

  // enable ADC and set ADC division factor to 128
  ADCSRA = (1 << ADATE) | ( 1 << ADEN ) | ( 1 << ADIE ) | 
  		   ( 1 << ADPS2 ) | ( 1 << ADPS1 ) | ( 1 << ADPS0 );
  ADC_out = 0;
  ADC_channel = CHANNEL0;
  ref_samples = 0;
}

void transmit(void){
	uint32_t rel;
	uint32_t dead;
	char dataM0;
	char dataM1;
	while(1) {
		//printf("Ref_Samples = %i\n", ref_samples);
		if (ref_samples == NUM_SAMPLES && gyro_updated)
		{
			dataM0 = avg_array(ADC_result0_avg, NUM_SAMPLES);
			dataM1 = avg_array(ADC_result1_avg, NUM_SAMPLES);
			if (dataM0 != data[6] || dataM1 != data[7])
			{
				data[6] = dataM0;
				data[7] = dataM1;
				printf("Send: xH=%i, xL=%i, yH=%i, yL=%i, zH=%i, zL=%i, 0=%i, 1=%i\n", data[0], data[1], data[2], data[3], data[4], data[5], data[6], data[7]);
				tx_me(data, data_length, tx_id);
				PORTD ^= (1<<7);
			}
			//printf("Send: xH=%i, xL=%i, yH=%i, yL=%i, zH=%i, zL=%i, 0=%i, 1=%i\n", data[0], data[1], data[2], data[3], data[4], data[5], data[6], data[7]);
				
			ref_samples = 0;
			gyro_updated = 0;
			ADCSRA |= (1 << ADSC); // start another conversion
		}
		rel = trtCurrentTime() + SECONDS2TICKS(.1);
		dead = trtCurrentTime() + SECONDS2TICKS(.1);
		trtSleepUntil(rel, dead);
	}
}

char avg_array(uint8_t a[], uint8_t num_elements)
{
	char i, sum = 0;
	for (i = 0; i < num_elements; i++)
	{
		sum = (sum + a[i])>>1;
	}
	//fprintf(stdout,"Average: %d\n", sum/num_elements);
	return sum;
}

void gyro (void)
{
	uint32_t rel;
	uint32_t dead;
	char xDataH = 0;
	char yDataH = 0;
	char zDataH = 0;
	char xDataM = 0;
	char yDataM = 0;
	char zDataM = 0;
	while(1)
	{
		if (!gyro_updated)
		{
			//Currently continuously updates the int values (if the high bits change) and prints them
			xDataM = gyro_get(GYRO_XOUT_H_REG);
			if (xDataH != xDataM)
			{
				xDataH = xDataM;
				data[0] = xDataH;
				data[1] = gyro_get(GYRO_XOUT_L_REG);
				//xData = (xDataH<<8) + gyro_get(GYRO_XOUT_L_REG);
				printf("xH = %x, xL = %x\n", data[0], data[1]);
				//data[0] = xData;
			}

			yDataM = gyro_get(GYRO_YOUT_H_REG);
			if (yDataH != yDataM)
			{
				yDataH = yDataM;
				data[2] = yDataH;
				data[3] = gyro_get(GYRO_YOUT_L_REG);
				//yData = (yDataH<<8) + gyro_get(GYRO_YOUT_L_REG);
				printf("yH = %x, yL = %x\n", data[2], data[3]);
				//data[1] = yData;
			}

			zDataM = gyro_get(GYRO_ZOUT_H_REG);
			if (zDataH != zDataM)
			{
				zDataH = zDataM;
				//zData = (zDataH<<8) + gyro_get(GYRO_ZOUT_L_REG);
				data[4] = zDataH;
				data[5] = gyro_get(GYRO_ZOUT_L_REG);
				printf("zH = %x, zL = %x\n", data[4], data[5]);
				//data[2] = zData;
			}
			gyro_updated = 1;
		}
	    rel = trtCurrentTime() + SECONDS2TICKS(.1);
	    dead = trtCurrentTime() + SECONDS2TICKS(.1);
	  	trtSleepUntil(rel, dead);
	}
}
//**********************************************************
// --- external interrupt ISR ------------------------
// Blinking indicator LEDs
unsigned char gyro_get(unsigned char address)
{
	unsigned char data;
	int i;
	TWCR &= ~(1<<TWEN);
	TWCR |= (1<<TWEN);
	
	//Send Start
	DDRC = DDRC | 0b00000010; // Port C2 is Output when writing
	TWCR = (1<<TWINT)|(1<<TWSTA)|(1<<TWEN);

	//printf("start sent\n");
	//Wait
	while (!(TWCR & (1<<TWINT)))// && (i < 1000))
	{i++;}
	i = 0;
	if((TWSR & 0xF8) != 0x08)
	{
		printf("Error at start %x\n", (TWSR & 0xF8));
		return 0;
	}
	//Send gyro write
	TWDR = ITG3200W;
	TWCR = (1<<TWINT)|(1<<TWEN);

	//printf("sent gyro write: TWDR = %x and write = %x and TWINT = %d\n", TWDR, ITG3200W, (TWCR & (1<<TWINT)));
	//Wait
	while (!(TWCR & (1<<TWINT)))// && (i < 1000))
	{ i++; }
	i = 0;
	if((TWSR & 0xF8) != 0x18)// && (TWSR & 0xF8) != 0x20)
	{
		printf("Error at write %x\n", TWSR);
		return 0;
	}
	//Send gyro ADDRESS
	TWDR = address;
	TWCR = (1<<TWINT)|(1<<TWEN);

	//printf("sent gyro ADDRESS:, %x\n", TWDR);
	//Wait
	while (!(TWCR & (1<<TWINT)))// && (i < 1000))
	{ i++; }
	i = 0;
	if((TWSR & 0xF8) != 0x28)// && (TWSR & 0xF8) != 0x30)
	{
		printf("Error at ADDRESS %x\n", TWSR);
		return 0;
	}
	//Send Start
	TWCR = (1<<TWINT)|(1<<TWSTA)|(1<<TWEN);
	while (!(TWCR & (1<<TWINT)));
	if((TWSR & 0xF8) != 0x10)
	{
		printf("Error at repeat start %x\n", TWSR);
		return 0;
	}
	
	//Send gyro read
	TWDR = ITG3200R;
	TWCR = (1<<TWINT)|(1<<TWEN);
	//printf("start sent and gyro read sent:, %x\n", TWDR);
	//Wait
	while (!(TWCR & (1<<TWINT)))// && (i < 1000))
	{ i++; }
	i = 0;
	if((TWSR & 0xF8) != 0x40 && (TWSR & 0xF8) != 0x48)
	{
		printf("Error at read %x\n", TWSR);
		return 0;
	}
	DDRC = DDRC & 0b11111101;
	//Receive data, Send ack/nack
	TWCR = (TWCR & 0x0F) | (1<<TWINT);// | (1<<TWEA);		
	//printf("sent ack/nack:, %x\n", TWCR);

	//Wait
	while (!(TWCR & (1<<TWINT)))// && (i < 1000))
	{ i++; }
	i = 0;
	if((TWSR & 0xF8) != 0x50 && (TWSR & 0xF8) != 0x58)
	{
		printf("Error at receive %x\n", TWSR);
		return 0;
	}

	//Get Data
	data = TWDR;
	//printf("waiting for data:, %x, %d\n", TWDR, data);
	//Wait
	while (!(TWCR & (1<<TWINT)))// && (i < 1000))
	{ i++; }
	i = 0;

	// STOP
	TWCR = (1<<TWINT)|(1<<TWEN)|(1<<TWSTO);

	//Disable, Enable 
	TWCR &= ~(1<<TWEN);
	TWCR |= (1<<TWEN);
	return data;
}

void i2cSetBitrate(unsigned short bitrateKHz)
{
	unsigned char bitrate_div;
	// Prescalar = 1
	// SCL freq = F_CPU/(16+2*TWBR))
	TWSR &= ~(1 << TWPS0); 
	TWSR &= ~(1 << TWPS1);
	
	//calculate bitrate division	
	bitrate_div = ((F_CPU/1000l)/bitrateKHz);
	if(bitrate_div >= 16)
		bitrate_div = (bitrate_div-16)/2;
	TWBR = bitrate_div; // 1st and 3rd bit on
}
//**********************************************************
// Displays the current sensor readings

int main(void)                                                              
{
  trt_uart_init();
  stdout = stdin = stderr = &uart_str;
  fprintf(stdout,"\n\r TRT \n\r\n\r");

  initialize();
  trtInitKernel(80); // 80 bytes for the idle task stack
  // --- create semaphores ----------
  // You must creat the first two semaphores if you use the uart
  trtCreateSemaphore(SEM_RX_ISR_SIGNAL, 0) ; // uart receive ISR semaphore
  trtCreateSemaphore(SEM_STRING_DONE,0) ;  // user typed <enter>
  
  // variable protection
  trtCreateSemaphore(SEM_SHARED, 1) ; // protect shared variables

 // --- creat tasks  ----------------
  //trtCreateTask(sensor_task, 1000, SECONDS2TICKS(0.05), SECONDS2TICKS(0.05),  &(args[0]));
  trtCreateTask(gyro, 1000, SECONDS2TICKS(0.05), SECONDS2TICKS(0.05), &(args[0]));
  trtCreateTask(transmit, 1000, SECONDS2TICKS(0.1), SECONDS2TICKS(0.1), &(args[0]));
  // --- Idle task --------------------------------------
  // just sleeps the cpu to save power 
  // every time it executes
  set_sleep_mode(SLEEP_MODE_IDLE);
  sleep_enable();
  while (1) 
  {
  	sleep_cpu();
  }
 } 
