/****************************************************************
 * Skyler Schneider ss868                                       *
 * Jack Ellsworth   jfe5                                        *
 * ECE 4760 Final Project                                       *
 * USB Wireless Tilt Mouse                                      *
 ****************************************************************/

// This is the txrx helper file created by Meghan Desai in 2005-2006.
// It was modified to suit the Mega644 by Skyler Schneider in 2010.

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>



#define MAX_LENGTH_RX      32
#define MAX_LENGTH_TX_PKT  32 
#define MAX_LENGTH_TX_DATA 16
#define synch_char 0b10101010  //170
#define start_char 0b10010101  //149 (used to be 202)
#define end_char   0b11010100  //212

// Encoding patterns - by transmitting one of these values,
// you can send a data nibble (4 bits, 16 possible values)
char code[16] = {
    0b10001011, // 0
    0b10001101, // 1
    0b10010011, // 2
    0b11001010, // 3 (used to be 149)
    0b10010110, // 4
    0b10011001, // 5
    0b10011010, // 6
    0b10011100, // 7
    0b10100011, // 8
    0b10100101, // 9
    0b10100110, // 10
    0b10101001, // 11
    0b10101100, // 12
    0b10110001, // 13
    0b10110010, // 14
    0b10110100  // 15
};

// General use variable
unsigned char txrx_i;

// RX variables
uint8_t rx_;
uint8_t decoded_byte;
uint8_t rx_byte;
uint8_t rx_done;
uint8_t rx_length;
uint8_t buffer;
uint8_t rx_started;
uint8_t max_rx_length;
uint8_t rx_led;
char    rx_data[MAX_LENGTH_RX];

// TX variables
uint8_t tx_data_length;
uint8_t tx_pkt_length;
uint8_t tx_id;
uint8_t tx_byte;
uint8_t tx_ing;
uint8_t tx_pkt_byte;
uint8_t tx_data_byte;
uint8_t tx_led;
char    tx_data[MAX_LENGTH_TX_PKT];
char    in_data[MAX_LENGTH_TX_DATA];

/****************************************************************
 * General Functions                                            *
 ****************************************************************/

// {tx,rx}
//   {0,0} none
//   {1,0} tx only
//   {0,1} rx only
//   {1,1} both
void txrx_init(int tx, int rx, int baud_num, char led) {
    // Set up the transmission
    char temp = 0;
    temp    = temp | (tx << 3) | (rx << 4) | (tx << 5) | (rx << 7);
    UCSR1B  = temp;
    UBRR1   = baud_num;    
    tx_byte = MAX_LENGTH_TX_PKT;
    tx_ing  = 0;
    tx_led  = led;
    rx_done = 1;
    rx_led  = led;
}

// Decode one transmitted byte into one data nibble
char decodeOne(char msbyte) {
    txrx_i = 0;
    buffer = start_char;
    while(txrx_i < 16) {
        if(msbyte == code[txrx_i]) {
            buffer = txrx_i;
        }
        txrx_i++;
    }
    return buffer;
}

// Decode two transmitted bytes into one data byte
char decode(char msbyte, char lsbyte) {
    txrx_i = 0;
    while(txrx_i < 16) {
        if(msbyte == code[txrx_i]) {
            buffer = txrx_i;
        }
        txrx_i++;
    }
    txrx_i=0;
    while(txrx_i < 16) {
        if(lsbyte == code[txrx_i]) {
            buffer = (buffer<<4);
            buffer = buffer | txrx_i; 
        }
        txrx_i++;
    }
    return buffer;
}


/****************************************************************
 * RX Functions                                                 *
 ****************************************************************/

ISR (USART1_RX_vect) {
    if(rx_done == 0)
    {
        if(rx_byte == 0) {
            rx_data[rx_byte] = UDR1;
            
            if(rx_data[0] == synch_char) rx_started = 1;
            else                         rx_started = 0;
            
            if(rx_data[0] == synch_char & rx_started) {
                rx_byte++;
                if(rx_led == 1) PORTD &= 0b01111111;
            }
        }
        else if(rx_byte == 1) {
            rx_data[rx_byte] = UDR1;
            if(rx_data[0] == synch_char && rx_data[1] == start_char) {
                rx_byte++;
            }
        }
        else {
            rx_data[rx_byte] = UDR1;
            if (rx_byte == 4) {
                rx_length = decode(rx_data[3], rx_data[4]);
            }
            else if((rx_byte >= max_rx_length) || (rx_data[rx_byte] == 212)) {
                if(rx_led==1) PORTD |= 0b10000000;
                rx_done=1;
            }
            rx_byte++;
        }
    }
}

void rx_reset(char max_rx) {
    rx_done       = 0;
    rx_started    = 0;
    rx_length     = 0;
    rx_byte       = 0;
    max_rx_length = max_rx;
}

char rxdone(void) { return rx_done; }


/****************************************************************
 * RX Iterator                                                  *
 ****************************************************************/

void init_getrx(void) { rx_ = 0; }
         
char get_next_rx_data(void) {
    if(rx_ <= 2) {
        return rx_data[rx_++];
    }
    else {
        decoded_byte = decode(rx_data[rx_], rx_data[(uint8_t)(rx_+1)]);
        rx_ = rx_ + 2;
        return decoded_byte;
    }
}

char rx_empty(void) { return (rx_ < max_rx_length) ? 0 : 1; }

/****************************************************************
 * TX Functions                                                 *
 ****************************************************************/

// UDR Empty ISR
ISR (USART1_UDRE_vect) {
    if(tx_byte < tx_pkt_length) {
        UDR1 = tx_data[tx_byte];
        tx_byte++;
        if(tx_led == 1) PORTC &= 0b01111111;
    }
    if(tx_byte == tx_pkt_length) {
        if(tx_led == 1) PORTC |= 0b10000000;
        tx_ing=0;
    }
}

void encode(void) {
    // Synchronization
    tx_data[0] = synch_char; // calibration
    tx_data[1] = synch_char; // calibration
    tx_data[2] = synch_char; // synchronization
    tx_data[3] = synch_char; // synchronization
    
    // Start character
    tx_data[4] = start_char; 
    
    // one byte for ID
    tx_data[5] = code[tx_id]; // Channel Id, static 0 for this unit
    
    // two bytes for length
    tx_data[6] = code[tx_data_length>>4];      // length of data to expect
    tx_data[7] = code[(tx_data_length<<4)>>4]; // length of data to expect
    
    // Payload
    tx_pkt_byte  = 8;
    tx_data_byte = 0; 
    while(tx_data_byte < tx_data_length) {
        tx_data[tx_pkt_byte] = code[in_data[tx_data_byte]>>4];
        tx_data[(uint8_t)(tx_pkt_byte+1)] = code[in_data[tx_data_byte] & 0x0F];
        tx_pkt_byte = tx_pkt_byte+2;
        tx_data_byte++;
    }
    
    // End character
    tx_data[tx_pkt_length-1] = end_char;
}             

void tx_me(char tx_data[], int length, int id) {
    if(tx_ing == 0) {
        txrx_i = 0;
        while(txrx_i < length) {
            in_data[txrx_i] = tx_data[txrx_i];
            txrx_i++;
        }
        tx_id = id;
        tx_data_length = length;
        tx_pkt_length  = length*2+9;
        encode();
        tx_byte = 0;
        tx_ing  = 1;
    }
}

char txdone(void) { return tx_ing; }
