/****************************************************************
 * Skyler Schneider ss868                                       *
 * Jack Ellsworth   jfe5                                        *
 * ECE 4760 Final Project                                       *
 * USB Wireless Tilt Mouse                                      *
 ****************************************************************/

// Receiver unit consists of USB and wireless RX.
// This file receives data wirelessly and sends it to the USB MCU.

/****************************************************************
 * Hardware Connections                                         *
 ****************************************************************/

/*
 * PORTB
 *  B.0 - Data to USB MCU
 *  B.1 - Data to USB MCU
 *  B.2 - Data to USB MCU
 *  B.3 - Data to USB MCU
 *  B.5 - Addr to USB MCU
 *  B.6 - Addr to USB MCU
 *  B.7 - Addr to USB MCU
 */

/*
 * PORTD
 *  D.0 - Wireless receiver
 *  D.5 - ACK from USB MCU
 *  D.6 - ACK from USB MCU
 *  D.7 - ACK from USB MCU
 */

/****************************************************************
 * Compiler Directives                                          *
 ****************************************************************/

#include <avr/io.h>
#include <avr/wdt.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <inttypes.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

// serial communication library
// Don't mess with the semaphores
#define SEM_RX_ISR_SIGNAL 1
#define SEM_STRING_DONE 2 // user hit <enter>
#include "trtSettings.h"
#include "trtUart.c"
#include "txrx_usart1_base.c"

// UART file descriptor
// putchar and getchar are in uart.c
FILE uart_str = FDEV_SETUP_STREAM(uart_putchar, uart_getchar, _FDEV_SETUP_RW);

// semaphore to protect shared variable
#define SEM_SHARED 3
int args[3] ;

// Wireless definitions
#define MAX_RX_LENGTH 32
#define data_buffer   32
#define threshold     2000

/****************************************************************
 * Global Declarations                                          *
 ****************************************************************/

// Structure for storing mouse status data to be sent to USB MCU
typedef struct {
    char buttonMask;
    char dxL;
    char dxH;
    char dyL;
    char dyH;
    char dWheelL;
    char dWheelH;
} txrx_usb_buf_t;

txrx_usb_buf_t rx_usb_buf;

// Wireless variables
char     my_rx_data[MAX_RX_LENGTH];
char     length;
uint16_t msSinceLastData;

// Timer1 variables for 1ms period
uint16_t TCNT1_prev;
uint16_t TCNT1_current;
uint16_t TCNT1_change;

    
int i;
int cntr = 0;
int msCntr = 0;
uint8_t receivedData;

void init(void); // initialization method
void receive(void); // receive function

/****************************************************************
 * Main Routine                                                 *
 ****************************************************************/

int main(void) {
	//init the UART -- trt_uart_init() is in trtUart.c
	trt_uart_init();
	stdout = stdin = stderr = &uart_str;
	fprintf(stdout,"\n\r TRT \n\r\n\r");

	init();
	// start TRT
	trtInitKernel(1000); // 80 bytes for the idle task stack
	// --- create semaphores ----------
	// You must creat the first two semaphores if you use the uart
	trtCreateSemaphore(SEM_RX_ISR_SIGNAL, 0) ; // uart receive ISR semaphore
	trtCreateSemaphore(SEM_STRING_DONE,0) ;  // user typed <enter>

	// variable protection
	trtCreateSemaphore(SEM_SHARED, 1) ; // protect shared variables

	// --- creat tasks  ----------------	  
	trtCreateTask(receive, 100, SECONDS2TICKS(0.05), SECONDS2TICKS(0.05), &(args[0]));

	// --- Idle task --------------------------------------
	// just sleeps the cpu to save power 
	// every time it executes
	set_sleep_mode(SLEEP_MODE_IDLE);
	sleep_enable();
	while (1) 
	{
		sleep_cpu();
	} 
}


            /*
            // 0 - 170
            // 1 - start_char
            // 2 - ID
            // 3 - length
            // 4 to 12 - payload
            rx_reset(MAX_RX_LENGTH);
            if(my_rx_data[4] == 6) {
                rx_usb_buf.buttonMask = 0b00100000 | (my_rx_data[5]  & 0x0F);
                rx_usb_buf.dxL        = 0b01000000 | (my_rx_data[6]  & 0x0F);
                rx_usb_buf.dxH        = 0b01100000 | (my_rx_data[7]  & 0x0F);
                rx_usb_buf.dyL        = 0b10000000 | (my_rx_data[8]  & 0x0F);
                rx_usb_buf.dyH        = 0b10100000 | (my_rx_data[9]  & 0x0F);
                rx_usb_buf.dWheelL    = 0b11000000 | (my_rx_data[10] & 0x0F);
                rx_usb_buf.dWheelH    = 0b11100000 | (my_rx_data[11] & 0x0F);
            }
            receivedData = 1;
        }
        
        TCNT1_current = TCNT1;
        TCNT1_change  = TCNT1_current - TCNT1_prev;
        if(TCNT1_change >= 250) {
            // Code runs every 1ms
            TCNT1_prev = TCNT1_current;
            msCntr++;
            if(receivedData) {
                msSinceLastData = 0;
            }
            else {
                msSinceLastData++;
                if(msSinceLastData == 1000) {
                    // If no packets received in past second, zero mouse
                    rx_usb_buf.buttonMask = 0b00100000 | 0x00;
                    rx_usb_buf.dxL        = 0b01000000 | 0x00;
                    rx_usb_buf.dxH        = 0b01100000 | 0x00;
                    rx_usb_buf.dyL        = 0b10000000 | 0x00;
                    rx_usb_buf.dyH        = 0b10100000 | 0x00;
                    rx_usb_buf.dWheelL    = 0b11000000 | 0x00;
                    rx_usb_buf.dWheelH    = 0b11100000 | 0x00;
                }
            }
            if(msCntr >= 500) {
                // Code runs every 500ms - blink LED at 1Hz
                msCntr = 0;
                PORTD = PORTD ^ 0b00000100;
            }
        }
        // Check if USB MCU has received data
        if((PORTB & 0b11100000) == (PIND & 0b11100000)) {
            switch(cntr) {
                case 0:
                    PORTB = rx_usb_buf.buttonMask;
                    cntr++;
                    break;
                case 1:
                    PORTB = rx_usb_buf.dxL;
                    cntr++;
                    break;
                case 2:
                    PORTB = rx_usb_buf.dxH;
                    cntr++;
                    break;
                case 3:
                    PORTB = rx_usb_buf.dyL;
                    cntr++;
                    break;
                case 4:
                    PORTB = rx_usb_buf.dyH;
                    cntr++;
                    break;
                case 5:
                    PORTB = rx_usb_buf.dWheelL;
                    cntr++;
                    break;
                case 6:
                    PORTB = rx_usb_buf.dWheelH;
                    cntr = 0;
                    break;
            }
        }
    }*/

/****************************************************************
 * Initialization Routine                                       *
 ****************************************************************/

void init() {
    
    // Set up ports
    DDRB = 0b11111111; // PORT B is an output
    //DDRD = 0b00000100; // LED
	DDRA = 0xFF; // PORT A is an output
    
    // Timer1 runs at prescalar 64
    TCCR1B = 0x03; // Timer1 runs at prescalar 64
    TCNT1_current = TCNT1;
    TCNT1_prev    = TCNT1_current;
    TCNT1_change  = 0;
    
    // Initialize variables
    rx_usb_buf.buttonMask = 0;
    rx_usb_buf.dxL        = 0;
    rx_usb_buf.dxH        = 0;
    rx_usb_buf.dyL        = 0;
    rx_usb_buf.dyH        = 0;
    rx_usb_buf.dWheelL    = 0;
    rx_usb_buf.dWheelH    = 0;
    
    // TXRX init
    txrx_init(0,1,249,0); // RX only - 4000 baud - led off
    rx_reset(MAX_RX_LENGTH);
    msSinceLastData = 0;
    
    // Enable interrupts
    sei();
}

/****************************************************************
 * Receive Routine				                                *
 ****************************************************************/
void receive(void) {
	// Check if received data from wireless receiver
	uint32_t rel;
	uint32_t dead;
	
	while(1) {
	    receivedData = 0;
		//fprintf(stdout,"helo\n");
	    if(rxdone() == 1) {
			//fprintf(stdout,"received\n");
			PORTA ^= 0x01; // blink first led;
			_delay_ms(50);
	        // Yes, received wireless data - decode it
	        i = 0;
	        init_getrx();
	        while(rx_empty() != 1) {
	            my_rx_data[i] = get_next_rx_data();
	            i++;
	        }
			//my_rx_data[0] = 'c';
			fprintf(stdout,"recevied data: %c\n", my_rx_data[0]);
		}
	}
	rel = trtCurrentTime() + SECONDS2TICKS(0.1);
	dead = trtCurrentTime() + SECONDS2TICKS(0.1);
	trtSleepUntil(rel, dead);
}
