/****************************************************************
 * Michael Wang (msw234)
 * John Fu (sf323)
 * Final Project ECE 4760 - Eyetracker
 ****************************************************************/

// Receiver unit consists of USB and wireless RX.
// This file receives data wirelessly and sends it to the USB MCU.

/****************************************************************
 * Hardware Connections                                         *
 ****************************************************************/

/*
 * PORTD
 *  D.2 - Wireless receiver
 *  D.2 - USART1
 *  D.3 - USART1
 *  A.0 - LED
 */

/****************************************************************
 * Compiler Directives                                          *
 ****************************************************************/

#include <avr/io.h>
#include <avr/wdt.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <inttypes.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "txrx_usart1_base.c"
#include "uart.h"
#include <avr/sleep.h>

// UART file descriptor
// putchar and getchar are in uart.c
FILE uart_str = FDEV_SETUP_STREAM(uart_putchar, uart_getchar, _FDEV_SETUP_RW);

// Wireless definitions
#define MAX_RX_LENGTH 32
#define data_buffer   32
#define threshold     2000
#define LED			  1
#define sensitivity 14.375
/****************************************************************
 * Global Declarations                                          *
 ****************************************************************/

// Wireless variables
char     my_rx_data[MAX_RX_LENGTH];
char     length;
uint16_t msSinceLastData;

int cntr = 0;
int msCntr = 0;

void init(void); // initialization method
void receive(void); // receive function

/****************************************************************
 * Main Routine                                                 *
 ****************************************************************/

int main(void) {
	init();
	while(1)
	{
		receive();
	}
}


/****************************************************************
 * Initialization Routine                                       *
 ****************************************************************/

void init() {
    // Set up ports
    //DDRB = 0b11111111; // PORT B is an output
    DDRA = 0b00000001; // LED	
	PORTA = 0b00000001;
	//PORTA |= (1 << LED); // Turn off led initially
    
    // TXRX init
    txrx_init(0,1,249,0); // RX only - 4000 baud - led off
    rx_reset(MAX_RX_LENGTH);
    msSinceLastData = 0;
    uart_init();
	stdout = stdin = stderr = &uart_str;
	fprintf(stdout,"\n\rHEX\n\r");

    // Enable interrupts
    sei();
}

/****************************************************************
 * Receive Routine				                                *
 ****************************************************************/
void receive(void) {
	// Check if received data from wireless receiver
	int16_t i, x, y, z;
	i = 0;
	x = 0;
	y = 0;
	z = 0; 
    if(rxdone() == 1) {
		//fprintf(stdout,"In rxdone\n");
		PORTA ^= 1; // blink first led;
        // Yes, received wireless data - decode it
        i = 0;
        init_getrx();
        while(rx_empty() != 1) {
            my_rx_data[i] = get_next_rx_data();
			//fprintf(stdout,"recevied data: i = %i, data = %i\n", i, my_rx_data[i]);
            i++;
        }
	    //printf("Header: %i,%i,%i ", my_rx_data[0], my_rx_data[1], my_rx_data[2]);
		//printf("Length: %i\n", my_rx_data[3]);
		x = ((my_rx_data[4]<<8) + my_rx_data[5])/sensitivity;
        y = ((my_rx_data[6]<<8) + my_rx_data[7])/sensitivity;
        z = ((my_rx_data[8]<<8) + my_rx_data[9])/sensitivity;
        printf("%i:%i:%i:%i:%i\n", x, y, z, my_rx_data[10], my_rx_data[11]);
		PORTA ^= 1;
		rx_reset(MAX_RX_LENGTH);
	}
}
