/****************************************************************
 * Skyler Schneider ss868                                       *
 * Jack Ellsworth   jfe5                                        *
 * ECE 4760 Final Project                                       *
 * USB Wireless Tilt Mouse                                      *
 ****************************************************************/

// Receiver unit consists of USB and wireless RX.
// This file receives data wirelessly and sends it to the USB MCU.

/****************************************************************
 * Hardware Connections                                         *
 ****************************************************************/

/*
 * PORTD
 *  D.0 - Wireless receiver
 *  D.1 - USART1
 *  D.2 - USART1
 *  D.4 - LED
 */

/****************************************************************
 * Compiler Directives                                          *
 ****************************************************************/

#include <avr/io.h>
#include <avr/wdt.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <inttypes.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

// serial communication library
// Don't mess with the semaphores
#define SEM_RX_ISR_SIGNAL 1
#define SEM_STRING_DONE 2 // user hit <enter>
#include "trtSettings.h"
#include "trtUart.c"
#include "txrx_usart1_base.c"

#include "trtkernel_1284.c"
#include <avr/sleep.h>

// UART file descriptor
// putchar and getchar are in uart.c
FILE uart_str = FDEV_SETUP_STREAM(uart_putchar, uart_getchar, _FDEV_SETUP_RW);

// semaphore to protect shared variable
#define SEM_SHARED 3
int args[3] ;

// Wireless definitions
#define MAX_RX_LENGTH 32
#define data_buffer   32
#define threshold     2000
#define LED			  1

/****************************************************************
 * Global Declarations                                          *
 ****************************************************************/

// Wireless variables
char     my_rx_data[MAX_RX_LENGTH];
char     length;
uint16_t msSinceLastData;

int i;
int cntr = 0;
int msCntr = 0;
uint8_t receivedData;

void init(void); // initialization method
void receive(void); // receive function

/****************************************************************
 * Main Routine                                                 *
 ****************************************************************/

int main(void) {
	//init the UART -- trt_uart_init() is in trtUart.c
	trt_uart_init();
	stdout = stdin = stderr = &uart_str;
	fprintf(stdout,"\n\r TRT \n\r\n\r");

	init();
	// start TRT
	trtInitKernel(1000); // 80 bytes for the idle task stack
	// --- create semaphores ----------
	// You must creat the first two semaphores if you use the uart
	trtCreateSemaphore(SEM_RX_ISR_SIGNAL, 0) ; // uart receive ISR semaphore
	trtCreateSemaphore(SEM_STRING_DONE,0) ;  // user typed <enter>

	// variable protection
	trtCreateSemaphore(SEM_SHARED, 1) ; // protect shared variables

	// --- creat tasks  ----------------	  
	trtCreateTask(receive, 1000, SECONDS2TICKS(0.1), SECONDS2TICKS(0.1), &(args[0]));

	// --- Idle task --------------------------------------
	// just sleeps the cpu to save power 
	// every time it executes
	set_sleep_mode(SLEEP_MODE_IDLE);
	sleep_enable();
	while (1) 
	{
		sleep_cpu();
	} 
}


/****************************************************************
 * Initialization Routine                                       *
 ****************************************************************/

void init() {
    // Set up ports
    DDRB = 0b11111111; // PORT B is an output
    DDRA = 0b00000001; // LED	

	PORTA |= (1 << 1); // Turn off led initially
    
    // TXRX init
    txrx_init(0,1,249,0); // RX only - 4000 baud - led off
    rx_reset(MAX_RX_LENGTH);
    msSinceLastData = 0;
    
    // Enable interrupts
    sei();
}

/****************************************************************
 * Receive Routine				                                *
 ****************************************************************/
void receive(void) {
	// Check if received data from wireless receiver
	uint32_t rel;
	uint32_t dead;
	
	while(1) {
	    receivedData = 0;
		PORTA ^= 1<<LED; // blink first led;
		//init_getrx();
		//fprintf(stdout,"In while\n");
	    if(rxdone() == 1) {
			fprintf(stdout,"In rxdone\n");
		
			//PORTD ^= (1<<LED); // blink first led;
			
	        // Yes, received wireless data - decode it
	        i = 0;
	        init_getrx();
	        while(rx_empty() != 1) {
	            my_rx_data[i] = get_next_rx_data();
				fprintf(stdout,"recevied data: %i\n", my_rx_data[i]);
	            i++;
	        }
			//my_rx_data[0] = 'c';
			//fprintf(stdout,"recevied data: %s\n", my_rx_data);
			//fprintf(stdout,"recevied data: %c\n", my_rx_data[0]);
			//fprintf(stdout,"recevied data: %c\n", my_rx_data[1]);
			//init_getrx();
			rx_reset(MAX_RX_LENGTH);
		}	
		rel = trtCurrentTime() + SECONDS2TICKS(0.1);
		dead = trtCurrentTime() + SECONDS2TICKS(0.1);
		trtSleepUntil(rel, dead);
	}
}
