/****************************************************************
 * Skyler Schneider ss868                                       *
 * Jack Ellsworth   jfe5                                        *
 * ECE 4760 Final Project                                       *
 * USB Wireless Tilt Mouse                                      *
 ****************************************************************/

// Transmitter unit consists of two accelerometers and wireless TX.

/****************************************************************
 * Hardware Connections                                         *
 ****************************************************************/

/*
 * PORTA
 *  A.0 - ADC Channel 0 (y Accelerometer)
 *  A.1 - ADC Channel 1 (x Accelerometer)
 */

/*
 * PORTC
 *  C.0 - Right  Pushbutton
 *  C.1 - Middle Pushbutton
 *  C.2 - Top    Pushbutton (Calibrate)
 *  C.3 - Left   Pushbutton
 */

/*
 * PORTD
 *  D.1 - Wireless transmitter
 */

/****************************************************************
 * Compiler Directives                                          *
 ****************************************************************/

#include <avr/io.h>
#include <avr/wdt.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/pgmspace.h>
#include <inttypes.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "txrx_usart0_head.c"

// Wireless definitions
#define data_length  1
#define tx_id        3
#define threshold    17

/****************************************************************
 * Global Declarations                                          *
 ****************************************************************/

// Wireless variables
char     data[data_length]; 
uint8_t  packet_count;
uint16_t time;

// ADC digital output
uint8_t  Ain;
uint8_t  measured;
uint8_t  measureMode; // 0 for y accelerometer, 1 for x accelerometer

// Timer1 variables for 16.384ms period
uint16_t TCNT1_prev;
uint16_t TCNT1_current;
uint16_t TCNT1_change;

void init(void); // initialization method

/****************************************************************
 * Interrupt Service Routines                                   *
 ****************************************************************/

ISR (TIMER0_COMPA_vect) {
    if(time <= threshold) time++;
}

/****************************************************************
 * Helper Functions                                             *
 ****************************************************************/

void generate_data(char count) {
    data[0] = 0x06;                       // IDENTIFIER BYTE
	data[1] = 0x06;
}

/****************************************************************
 * Main Routine                                                 *
 ****************************************************************/
int main(void) {
    int8_t tempInt;
    init();
  
    while(1) {
        // Send wireless information every 17ms
        if(time == threshold) {
            // Executes every threshold ms
            generate_data(packet_count);			
            tx_me(data, data_length, tx_id);
            time = 0;
            packet_count++;
        }
    }
}

/****************************************************************
 * Initialization Routine                                       *
 ****************************************************************/

void init() {
    
    // Set up ports
    DDRA = 0x00; // ADC inputs
    DDRC = 0x00; // pushbutton inputs
    DDRD = 0xFF; // wireless tx output
    
    // Set up Timer0
    TIMSK0 = (1<<OCIE0A); // Turn on Timer0 comp match ISR
    OCR0A  = 250;         // Set the compare reg to 250 time ticks (1ms)
    // Prescalar to 64 and turn on clear-on-match
    TCCR0A = (1<<WGM01);  // Turn on clear-on-match
    TCCR0B = 0x03;        // Clock prescalar to 64
    
    // Set up wireless transmitter
    txrx_init(1,0,249,0); // TX only - 4000 baud - led off
    
	time = 0;
    packet_count = 0;
    generate_data(packet_count);
    
    // Enable interrupts
    sei();
}
